
#ifdef CLEANMEM_DISABLE
#	undef malloc
#	undef free
#	define free(x) customFreeEnabled = false, free(x)
#	undef new
#	undef delete
#	define delete customFreeEnabled = false, delete
#endif


#ifndef CLEANMEM_INCLUDED
#define CLEANMEM_INCLUDED

#include "gamex-global-def.h"




#ifdef CLEANMEM_DISABLE

	extern void BeginCheckingUnfreed();
	extern void DumpUnfreed();
	extern bool checkLeakScope;
	extern bool customFreeEnabled;

#else // !CLEANMEM_DISABLE:


// we'll get errors compiling if these are included after new is redefined, so always include them beforehand
// (this is because arena allocation has such weird syntax that no possible definition of new can avoid a syntax error with it
//  if only new and delete weren't such special-case things syntactically, this would all be much easier...)
#include <string>
#include <list>
#include <xlocale>
#include <map>
#include <queue>

// a vector is used to keep track of allocations (and we probably have to include it here anyway)
#include <vector>


//  #include <condefs.h>
//  #include <iostream>
//  #include <fstream>
//  #include <iomanip.h>

#pragma warning(disable: 4311)
#pragma warning(disable: 4267)
#pragma warning(disable: 4291)



	// make sure that there's a filename specified for the log file...
#	ifndef CLEAN_MEM_LOG_FILE
#		define CLEAN_MEM_LOG_FILE "leaks.log"
#	endif

	struct AllocInfo;
	typedef std::vector<AllocInfo> AllocList;
	extern AllocList *allocList;
	extern const char * allocDeleteFile;
	extern int allocDeleteLine;
	extern void BeginCheckingUnfreed();
	extern void DumpUnfreed();
	extern void AddTrack(unsigned long addr,  unsigned long asize,  const char *fname, unsigned long lnum, bool array);
	extern bool RemoveTrack(unsigned long addr, bool array);
	extern void * operator new(size_t size, const char *file, int line);
	extern void * operator new[](size_t size, const char *file, int line);
	extern void operator delete(void* mem);
	extern void operator delete[](void* mem);
	extern void * __malloc(size_t size, const char *file, int line);
	extern void __free(void * mem);
	extern bool checkLeakScope;
	extern bool customFreeEnabled;
	extern bool leakDebugFileOpened;

	// macro stuff for putting in file & line #

#	define new new(__FILE__, __LINE__)

#	ifndef CLEAN_MEM_NO_VERIFY

		// for some reason delete cannot be overrided as easily as new can
		// so here's a tricky macro to pass the information anyway
#		define delete allocDeleteLine = __LINE__, allocDeleteFile = __FILE__, delete

#	endif // !CLEAN_MEM_NO_VERIFY


#	define malloc(s) __malloc(s, __FILE__, __LINE__)
#	define free(p) allocDeleteLine = __LINE__, allocDeleteFile = __FILE__, __free(p)



#endif // !CLEANMEM_DISABLE


#endif // !CLEANMEM_INCLUDED
